%% Read Vernan documentation
help Vernan

%% Set-up inputs
n = 10;
Q = cell(10,3);

%   Xj                Yj                Zj
Q{1,1} = 1;       Q{1,2} = 2;       Q{1,3} = 1;
Q{2,1} = [2,9];   Q{2,2} = [3,10];  Q{2,3} = [2,1];
Q{3,1} = [3,10];  Q{3,2} = [2,9];   Q{3,3} = [2,1];
Q{4,1} = 2;       Q{4,2} = 1;       Q{4,3} = 1;
Q{5,1} = 4;       Q{5,2} = 5;       Q{5,3} = 1;
Q{6,1} = 5;       Q{6,2} = 4;       Q{6,3} = 1;
Q{7,1} = 6;       Q{7,2} = 7;       Q{7,3} = 2;
Q{8,1} = 7;       Q{8,2} = 6;       Q{8,3} = 2;
Q{9,1} = 8;       Q{9,2} = 9;       Q{9,3} = 1;
Q{10,1} = 9;      Q{10,2} = 8;      Q{10,3} = 1;

finit = [1,1,4,4,6,6,8];

rho = cell(1,8);
rho{1} = [1,2];  rho{2} = 3;  rho{3} = 4;  rho{4} = 5;
rho{5} = 6;      rho{6} = 7;  rho{7} = [8,9]; rho{8} = 10;
 
%Dependency graphs below are indexed according to the order the graphs are 
%listed in Table 5
D = cell(9,1);
D{1} = cell(1,3);  D{2} = cell(2,3);  D{3} = cell(2,3);  D{4} = cell(2,3);
D{5} = cell(2,3);  D{6} = cell(1,3);  D{7} = cell(1,3);  D{8} = cell(2,3);
D{9} = cell(1,3);

%        Ay                   By                   Ry
D{1}{1,1} = [1,2,7];  D{1}{1,2} = [5,6];    D{1}{1,3} = [5,5];
%---------------------------------------------------------------
D{2}{1,1} = 3;        D{2}{1,2} = [1,2,7];  D{2}{1,3} = [1,1,7];
D{2}{2,1} = 4;        D{2}{2,2} = [1,2,7];  D{2}{2,3} = [1,1,7];
%---------------------------------------------------------------
D{3}{1,1} = 3;        D{3}{1,2} = [1,2,7];  D{3}{1,3} = [2,2,8];
D{3}{2,1} = 4;        D{3}{2,2} = [1,2,7];  D{3}{2,3} = [2,2,8];
%---------------------------------------------------------------
D{4}{1,1} = 3;        D{4}{1,2} = [5,6];    D{4}{1,3} = [5,5];
D{4}{2,1} = 4;        D{4}{2,2} = [5,6];    D{4}{2,3} = [5,5];
%---------------------------------------------------------------
D{5}{1,1} = 3;        D{5}{1,2} = [5,6];    D{5}{1,3} = [6,6];
D{5}{2,1} = 4;        D{5}{2,2} = [5,6];    D{5}{2,3} = [6,6];
%---------------------------------------------------------------
D{6}{1,1} = [5,6];    D{6}{1,2} = [1,2,7];  D{6}{1,3} = [1,1,7];
%---------------------------------------------------------------
D{7}{1,1} = [5,6];    D{7}{1,2} = [3,4];    D{7}{1,3} = [3,3];
%---------------------------------------------------------------
D{8}{1,1} = [5,6];    D{8}{1,2} = [3,4];    D{8}{1,3} = [3,4];
D{8}{2,1} = [5,6];    D{8}{2,2} = [3,4];    D{8}{2,3} = [4,3];
%---------------------------------------------------------------
D{9}{1,1} = [5,6];    D{9}{1,2} = [3,4];    D{9}{1,3} = [4,4];

rules = zeros(14,4);
rules(1,:) = [1,1,0,0];  rules(2,:) = [2,2,1,0];  rules(3,:) = [3,3,0,0];
rules(4,:) = [4,4,0,0];  rules(5,:) = [5,5,2,0];  rules(6,:) = [5,6,3,0];
rules(7,:) = [6,7,4,0];  rules(8,:) = [6,8,5,0];  rules(9,:) = [7,9,6,0];
rules(10,:) = [8,10,7,0];  rules(11,:) = [8,11,8,0];  
rules(12,:) = [8,12,9,0];  rules(13,:) = [9,13,0,0];
rules(14,:) = [10,14,0,0];

%% Generate state-space graphs
symred = 'off'; %generate full model
[Ffull,Tfull,Wfull] = Vernan(n,Q,rules,D,rho,finit,symred);

symred = 'on'; %generate reduced model
[Fred,Tred,Wred,M] = Vernan(n,Q,rules,D,rho,finit,symred);

%% Solve ODEs
help SolveMasterEqtn_transient %read documentation

C = cell(14,1); % a set of 14 specific probability rate constants
for i = [1:4,7:14]
    C{i} = 10*rand(1);
end
k5 = 10*rand(1);
k6 = (10 - k5)*rand(1) + k5;
C{5} = @(t)k5 + 5*(1+sin(2*pi*t)); %C{5} and C{6} are made time-dependent
C{6} = @(t)k6 + 5*(1+sin(2*pi*t));

odesolver = @ode15s; % solve using MATLAB's built-in ode15s solver
tspan = [0,5]; % integrate over 5 seconds
options = odeset('RelTol',1e-12); % set tolerance

Pfull0 = zeros(length(Ffull),1);
Jfull = ismember(Ffull,finit,'rows'); % locates index of finit in Ffull
Pfull0(Jfull) = 1;

Pred0 = zeros(length(Fred),1);
Jred = ismember(Fred,finit,'rows'); % locates index of finit in Fred
Pred0(Jred) = 1;

[Pfull,tfull] = SolveMasterEqtn_transient(Ffull,Tfull,Wfull,1,C, ...
                odesolver,tspan,Pfull0,options); %solve full model
            
[Pred,tred] = SolveMasterEqtn_transient(Fred,Tred,Wred,M,C, ...
              odesolver,tspan,Pred0,options); %solve reduced model

%% Plot results
%plot time-evolution of open states
Jfull = ismember(Ffull(:,7),10); %to find open states, we locate the states
Jred = ismember(Fred(:,7),10);   %with component 7 in state 10
Pfull_open = sum(Pfull(Jfull,:),1);
Pred_open = sum(Pred(Jred,:),1);

figure(1); % corresponds to Figure 5A in text
plot(tfull,Pfull_open,'r',tred,Pred_open,':k');
xlabel('time (seconds)');
ylabel('Probability');
 
%plot time-evolution of state f = (3,3,5,4,6,6,10)
Jfull = ismember(Ffull,[3,3,5,4,6,6,10],'rows');
Jred = ismember(Fred,[3,3,4,5,6,6,10],'rows');
Pfull_f = Pfull(Jfull,:);
Pred_ftilde = Pred(Jred,:);
Pred_f = Pred(Jred,:)./2;

figure(2); % corresponds to Figure 5B in text
plot(tfull,Pfull_f,'r',tred,Pred_ftilde,'k',tred,Pred_f,':k');
xlabel('time (seconds)');
ylabel('Probability');
